@@ -2,7 +2,7 @@
    authmode.c
 
    This file is part of GNU Anubis.
-   Copyright (C) 2003, 2004, 2005, 2007, 2008 The Anubis Team.
+   Copyright (C) 2003-2020 The Anubis Team.
 
    GNU Anubis is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -25,7 +25,7 @@
 #ifdef WITH_GSASL
 
 static char *smtp_greeting_message;
-static ANUBIS_LIST *smtp_help_message;
+static ANUBIS_LIST smtp_help_message;
 static int helo_count; /* report possible SMTP attack */
 
 
@@ -68,12 +68,7 @@ enum asmtp_state
 static int
 asmtp_kw (const char *name)
 {
-  static struct kw
-  {
-    char *name;
-    int code;
-  }
-  kw[] = {
+  static struct anubis_keyword kw[] = {
     { "helo", KW_HELO },
     { "ehlo", KW_EHLO },
     { "xelo", KW_XELO },
@@ -86,14 +81,9 @@ asmtp_kw (const char *name)
     { "xdatabase", KW_XDATABASE },
     { NULL },
   };
-  int i;
-
-  if (name)
-    {
-      for (i = 0; kw[i].name != NULL; i++)
-	if (strcasecmp (name, kw[i].name) == 0)
-	  return kw[i].code;
-    }
+  struct anubis_keyword *kwp = anubis_keyword_lookup_ci (kw, name);
+  if (kwp)
+    return kwp->tok;
   return -1;
 }
 
@@ -135,7 +125,7 @@ asmtp_greet (void)
   asmtp_reply (220, "%s %s", name, smtp_greeting_message);
 }
 
-static ANUBIS_LIST *asmtp_capa;
+static ANUBIS_LIST asmtp_capa;
 
 void
 asmtp_capa_add (char *name)
@@ -181,7 +171,7 @@ asmtp_capa_init (void)
 static void
 asmtp_capa_report (void)
 {
-  ITERATOR *itr = iterator_create (asmtp_capa);
+  ITERATOR itr = iterator_create (asmtp_capa);
   char *p = iterator_first (itr);
 
   while (p)
@@ -230,7 +220,7 @@ asmtp_ehlo_reply (char *args)
       return state_init;
     }
   
-  set_ehlo_domain(domain);
+  set_ehlo_domain (domain, strlen (domain));
   
   helo_count++;
   asmtp_reply (R_CONT | 250, "Anubis is pleased to meet you.");
@@ -268,7 +258,7 @@ asmtp_help (void)
   if (smtp_help_message)
     {
       char *s;
-      ITERATOR *itr = iterator_create (smtp_help_message);
+      ITERATOR itr = iterator_create (smtp_help_message);
       for (s = iterator_first (itr); s; s = iterator_next (itr))
 	asmtp_reply (R_CONT | 214, "%s", s);
       iterator_destroy (&itr);
@@ -364,11 +354,7 @@ asmtp_ehlo (enum asmtp_state state, ANUBIS_USER * usr)
 	  }
 
 	asmtp_reply (220, "Ready to start TLS");
-	stream = start_ssl_server (remote_client,
-				   secure.cafile,
-				   secure.cert,
-				   secure.key,
-				   options.termlevel > NORMAL);
+	stream = start_ssl_server (remote_client, options.termlevel > NORMAL);
 	if (!stream)
 	  {
 	    asmtp_reply (454, "TLS not available" CRLF);
@@ -515,7 +501,7 @@ anubis_get_db_record (const char *username, ANUBIS_USER * usr)
 {
   void *db;
   int rc;
-  char *errtext;
+  char const *errtext;
 
   if (!anubis_dbarg)
     {
@@ -624,61 +610,64 @@ anubis_authenticate_mode (struct sockaddr_in *addr)
 #define KW_SMTP_GREETING_MESSAGE 6
 #define KW_SMTP_HELP_MESSAGE     7
 
-static int
-rc_parser (int method, int key, ANUBIS_LIST * arglist,
-	   void *inv_data, void *func_data, MESSAGE * msg)
+static void
+rc_parser (EVAL_ENV env, int key, ANUBIS_LIST arglist, void *inv_data)
 {
   char *arg = list_item (arglist, 0);
 
-  switch (key) {
-  case KW_SMTP_GREETING_MESSAGE:
-    if (list_count (arglist) != 1)
-      return RC_KW_ERROR;
-    xfree (smtp_greeting_message);
-    smtp_greeting_message = strdup (arg);
-    break;
-    
-  case KW_SMTP_HELP_MESSAGE:
-    if (list_count (arglist) != 1)
-      return RC_KW_ERROR;
-    make_help_message (arg);
-    break;
-    
-  case KW_SASL_PASSWORD_DB:
-    if (list_count (arglist) != 1)
-      return RC_KW_ERROR;
-    anubis_set_password_db (arg);
-    break;
-    
-  case KW_SASL_ALLOWED_MECH:
-    anubis_set_server_mech_list (arglist);
-    break;
-
-  case KW_SASL_SERVICE:
-    if (list_count (arglist) != 1)
-      return RC_KW_ERROR;
-    xfree (anubis_sasl_service);
-    anubis_sasl_service = strdup (arg);
-    break;
-    
-  case KW_SASL_REALM:
-    if (list_count (arglist) != 1)
-      return RC_KW_ERROR;
-    xfree (anubis_sasl_realm);
-    anubis_sasl_realm = strdup (arg);
-    break;
-    
-  case KW_SASL_HOSTNAME:
-    if (list_count (arglist) != 1)
-      return RC_KW_ERROR;
-    xfree (anubis_sasl_hostname);
-    anubis_sasl_hostname = strdup (arg);
-    break;
-    
-  default:
-    return RC_KW_UNKNOWN;
-  }
-  return RC_KW_HANDLED;
+  switch (key)
+    {
+    case KW_SMTP_GREETING_MESSAGE:
+      if (list_count (arglist) != 1)
+	eval_error (0, env, _("invalid number of arguments"));
+      xfree (smtp_greeting_message);
+      smtp_greeting_message = strdup (arg);
+      break;
+      
+    case KW_SMTP_HELP_MESSAGE:
+      if (list_count (arglist) != 1)
+	eval_error (0, env, _("invalid number of arguments"));
+      make_help_message (arg);
+      break;
+      
+    case KW_SASL_PASSWORD_DB:
+      if (list_count (arglist) != 1)
+	eval_error (0, env, _("invalid number of arguments"));
+      anubis_set_password_db (arg);
+      break;
+      
+    case KW_SASL_ALLOWED_MECH:
+      anubis_set_server_mech_list (arglist);
+      break;
+      
+    case KW_SASL_SERVICE:
+      if (list_count (arglist) != 1)
+	eval_error (0, env, _("invalid number of arguments"));
+      xfree (anubis_sasl_service);
+      anubis_sasl_service = strdup (arg);
+      break;
+      
+    case KW_SASL_REALM:
+      if (list_count (arglist) != 1)
+	eval_error (0, env, _("invalid number of arguments"));
+      xfree (anubis_sasl_realm);
+      anubis_sasl_realm = strdup (arg);
+      break;
+      
+    case KW_SASL_HOSTNAME:
+      if (list_count (arglist) != 1)
+	eval_error (0, env, _("invalid number of arguments"));
+      xfree (anubis_sasl_hostname);
+      anubis_sasl_hostname = strdup (arg);
+      break;
+      
+    default:
+      eval_error (2, env,
+		  _("INTERNAL ERROR at %s:%d: unhandled key %d; "
+		    "please report"),
+		  __FILE__, __LINE__,
+		  key);
+    }
 }
 
 static struct rc_kwdef init_authmode_kw[] = {
