@@ -1,6 +1,6 @@
 /*
    GNU Anubis -- an SMTP message submission daemon.
-   Copyright (C) 2001-2014 The Anubis Team.
+   Copyright (C) 2001-2020 The Anubis Team.
 
    GNU Anubis is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -98,7 +98,7 @@ main (int argc, char *argv[])
 #ifdef WITH_GSASL
   dbtext_init ();
 # ifdef HAVE_LIBGDBM
-  gdbm_db_init ();
+  db_gdbm_init ();
 # endif
 # ifdef WITH_MYSQL
   mysql_db_init ();
