@@ -2,8 +2,7 @@
    rcfile.c
 
    This file is part of GNU Anubis.
-   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2007, 2009,
-   2011 The Anubis Team.
+   Copyright (C) 2001-2020 The Anubis Team.
 
    GNU Anubis is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -329,22 +328,6 @@ static volatile unsigned long _anubis_hang;
 /* List of users who are allowed to use HANG in their profiles */
 ANUBIS_LIST allow_hang_users; 
 
-static struct rc_kwdef esmtp_kw[] = {
-  { "esmtp-auth",   KW_ESMTP_AUTH, KWF_HIDDEN },
-  { "esmtp-anonymous-token", KW_ESMTP_ANONYMOUS_TOKEN, KWF_HIDDEN },
-  { "esmtp-auth-id", KW_ESMTP_AUTH_ID, KWF_HIDDEN },
-  { "esmtp-authz-id", KW_ESMTP_AUTHZ_ID, KWF_HIDDEN },
-  { "esmtp-password", KW_ESMTP_PASSWORD, KWF_HIDDEN },
-  { "esmtp-service",  KW_ESMTP_SERVICE, KWF_HIDDEN },
-  { "esmtp-hostname", KW_ESMTP_HOSTNAME, KWF_HIDDEN },
-  { "esmtp-generic-service", KW_ESMTP_SERVICE, KWF_HIDDEN },
-  { "esmtp-passcode", KW_ESMTP_PASSCODE, KWF_HIDDEN },
-  { "esmtp-realm", KW_ESMTP_REALM, KWF_HIDDEN },
-  { "esmtp-allowed-mech", KW_ESMTP_ALLOWED_MECH },
-  { "esmtp-require-encryption", KW_ESMTP_REQUIRE_ENCRYPTION },
-  { NULL }
-};
-
 static int
 parse_esmtp_kv (int key, ANUBIS_LIST arglist)
 {
@@ -574,7 +557,7 @@ control_parser (EVAL_ENV env, int key, ANUBIS_LIST arglist, void *inv_data)
 	  {
 	    *p++ = 0;
 	    assign_string (&session.socks_password, p);
-	    assign_string (session.socks_username, arg);
+	    assign_string (&session.socks_username, arg);
 	    topt |= T_SOCKS_AUTH;
 	  }
 	break;
@@ -765,6 +748,7 @@ static struct rc_secdef_child control_sect_child = {
 #define KW_SSL_CERT            3
 #define KW_SSL_KEY             4
 #define KW_SSL_CAFILE          5
+#define KW_SSL_PRIORITIES      6
 
 void
 tls_parser (EVAL_ENV env, int key, ANUBIS_LIST arglist, void *inv_data)
@@ -796,6 +780,11 @@ tls_parser (EVAL_ENV env, int key, ANUBIS_LIST arglist, void *inv_data)
       xfree (secure.cafile);
       secure.cafile = xstrdup (arg);
       break;
+
+    case KW_SSL_PRIORITIES:
+      xfree (secure.prio);
+      secure.prio = xstrdup (arg);
+      break;
       
     default:
       eval_error (2, env,
@@ -807,11 +796,12 @@ tls_parser (EVAL_ENV env, int key, ANUBIS_LIST arglist, void *inv_data)
 }
 
 static struct rc_kwdef tls_kw[] = {
-  { "ssl",        KW_SSL },
-  { "ssl-oneway", KW_SSL_ONEWAY },
-  { "ssl-cert",   KW_SSL_CERT },
-  { "ssl-key",    KW_SSL_KEY },
-  { "ssl-cafile", KW_SSL_CAFILE },
+  { "ssl",            KW_SSL },
+  { "ssl-oneway",     KW_SSL_ONEWAY },
+  { "ssl-cert",       KW_SSL_CERT },
+  { "ssl-key",        KW_SSL_KEY },
+  { "ssl-cafile",     KW_SSL_CAFILE },
+  { "ssl-priorities", KW_SSL_PRIORITIES },
   { NULL }
 };
 
