@@ -2,7 +2,7 @@
    xdatabase.c
 
    This file is part of GNU Anubis.
-   Copyright (C) 2004, 2007 The Anubis Team.
+   Copyright (C) 2004-2020 The Anubis Team.
 
    GNU Anubis is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -34,34 +34,10 @@ xdatabase_enable ()
 }
 
 void
-xdatabase_capability (char **preply, size_t *preply_size)
+xdatabase_capability (ANUBIS_SMTP_REPLY reply)
 {
-  static char capa_string[] = "250-XDATABASE\r\n";
-  size_t capa_len = strlen (capa_string);
-  size_t len;
-  char *p;
-  char *reply;
-  
-  if (!xdatabase_active)
-    return;
-
-  if (strlen (*preply) + capa_len >= *preply_size)
-    {
-      *preply_size = strlen (*preply) + capa_len + 1;
-      *preply = xrealloc (*preply, *preply_size);
-    }
-  reply = *preply;
-  
-  p = strstr (reply, "250 ");
-  if (!p)
-    {
-      anubis_error (0, 0,
-		    _("Cannot add capability: input string missing end marker"));
-      return;
-    }
-  len = strlen (p);
-  memmove (p + capa_len, p, len);
-  memmove (p, capa_string, capa_len);
+  if (!smtp_reply_has_capa (reply, "XDATABASE", NULL))
+    smtp_reply_add_line (reply, "XDATABASE");
 }
 
 static FILE *
@@ -101,7 +77,8 @@ make_temp_file (struct obstack *stk, char *rcname, char **name)
 
 static void
 _xdb_error_printer (void *data,
-		    const char *filename, int line,
+		    struct rc_loc *loc,
+		    const char *pfx,
 		    const char *fmt, va_list ap)
 {
   struct obstack *stk = data;
@@ -109,8 +86,20 @@ _xdb_error_printer (void *data,
   int n;
 
   obstack_grow (stk, ERROR_PREFIX, sizeof ERROR_PREFIX - 1);
-  n = snprintf (buf, sizeof buf, "%d: ", line);
+  /* FIXME: column? */
+  n = snprintf (buf, sizeof buf, "%lu", (unsigned long)loc->line);
   obstack_grow (stk, buf, n);
+  if (topt & T_LOCATION_COLUMN)
+    {
+      n = snprintf (buf, sizeof buf, ".%lu", (unsigned long)loc->column);
+      obstack_grow (stk, buf, n);
+    }
+  obstack_grow (stk, ": ", 2);
+  if (pfx)
+    {
+      obstack_grow (stk, pfx, strlen (pfx));
+      obstack_grow (stk, ": ", 2);
+    }
   n = vsnprintf (buf, sizeof buf, fmt, ap);
   obstack_grow (stk, buf, n);
   obstack_grow (stk, CRLF, 2);
@@ -229,7 +218,7 @@ xexamine ()
       memset (hex, 0, sizeof hex); 
       string_bin_to_hex (hex, digest, sizeof digest);
       swrite (SERVER, remote_client, "250 ");
-      swrite (SERVER, remote_client, hex);
+      swrite (SERVER, remote_client, (char*) hex);
       swrite (SERVER, remote_client, CRLF);
     }
   free (rcname);
